package com.ruoyi.domain;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.ruoyi.common.StandardJsonTypeHandler;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.Date;
import java.util.List;

/**
 * 车型试验任务
 * @author gxk
 */
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
@ApiModel("车型试验任务")
@TableName(value ="t_model_test_task")
public class ModelTestTask {

    /**
     * 任务类型
     */
    @TableField(exist = false)
    public static final String TYPE = "test";

    /**
     * 任务状态
     */
    @TableField(exist = false)
    public static final String TASK_STATUS_NEW = "NEW";
    @TableField(exist = false)
    public static final String TASK_STATUS_PENDING = "PENDING";
    @TableField(exist = false)
    public static final String TASK_STATUS_SIGNED = "SIGNED";
    @TableField(exist = false)
    public static final String TASK_STATUS_FINISH = "FINISH";

    @ApiModelProperty("主键")
    @TableId(type = IdType.ASSIGN_ID)
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    private Long id;

    @ApiModelProperty("任务编号")
    private String taskNo;

    @ApiModelProperty("任务状态[NEW、PENDING、SIGNED、FINISH]")
    private String taskStatus;

    @ApiModelProperty("任务发起人姓名")
    private String taskInitiator;

    @ApiModelProperty("任务发起人部门")
    private String taskInitiatorDept;

    @ApiModelProperty("任务结果（PASS、REJECT）")
    private String taskResult;

    @ApiModelProperty("任务开始时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm")
    private Date taskBeginTime;

    @ApiModelProperty("任务结束时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm")
    private Date taskEndTime;

    @ApiModelProperty("测试用例")
    @TableField(value="test_case", typeHandler = StandardJsonTypeHandler. class)
    private List<ReviewStandard> testCase;

    @ApiModelProperty("创建时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm")
    private Date createTime;

    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @ApiModelProperty("审查组长id")
    private Long leaderId;

    @ApiModelProperty("审查组长姓名")
    private String leader;

    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @ApiModelProperty("关联标准表id")
    private Long standardId;

    @ApiModelProperty("标准名称")
    private String name;

    @ApiModelProperty("标准号")
    private String standardNo;

    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @ApiModelProperty("提交人id")
    private Long submitId;

    @ApiModelProperty("提交人姓名")
    private String submitName;

    @ApiModelProperty("签名图片地址")
    private String imagesUrl;

    @ApiModelProperty("车型试验测试用例数")
    private Integer details;

    @ApiModelProperty("车型试验标准条款数")
    private Integer standards;

    @ApiModelProperty("试验方案ID")
    private String testSchemeId;

}
