package com.ruoyi.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.ruoyi.common.DataStatisticsUtils;
import com.ruoyi.common.constant.HttpStatus;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.domain.*;
import com.ruoyi.mapper.*;
import com.ruoyi.service.ReviewDetailsResultService;
import com.ruoyi.service.ReviewSceneChangeTaskService;
import com.ruoyi.service.ReviewStandardService;
import com.ruoyi.service.StrategyCarReviewTask;
import com.ruoyi.web.response.CarReviewTaskViewResponse;
import com.ruoyi.web.response.DataStatisticsResponse;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.swing.table.TableModel;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.ruoyi.common.DataStatisticsUtils.calculateStandardsPass;

@Transactional
@Service
public class StrategyCarReviewTaskSigned implements StrategyCarReviewTask, InitializingBean {

    @Autowired
    private StrategyCarReviewTaskContext strategyCarReviewTaskContext;

    @Autowired
    private ApplicationContext applicationContext;

    @Autowired
    private CarReviewTaskMapper carReviewTaskMapper;

    @Autowired
    private TaskMapper taskMapper;

    @Autowired
    private TaskStateFacade taskStateFacade;

    @Autowired
    private ReviewStandardService reviewStandardService;

    @Autowired
    private ReviewDetailsResultService reviewDetailsResultService;

    @Autowired
    private TaskUserRelationMapper taskUserRelationMapper;

    @Autowired
    private TaskScenarioRelationMapper taskScenarioRelationMapper;

    @Autowired
    private TestScenarioMapper testScenarioMapper;

    // SIGNED状态下
    // 1.standard字段使用task中归档的问卷
    // 2.不查询场景变更任务
    // 3.无需联查问卷答案
    @Override
    public CarReviewTaskViewResponse doView(CarReviewTask carReviewTask) {
        CarReviewTaskViewResponse response = new CarReviewTaskViewResponse();
        carReviewTask.setImagesUrl("/vehicle-quality-review-oss/2024/03/04/qianzi1_20240304104011A001.jpg,/vehicle-quality-review-oss/2024/03/04/qianzi2_20240304104121A002.jpg");

        // 获取问卷
        List<ReviewStandard> carStandardTree = reviewStandardService.findTree(carReviewTask.getStandardId(), ReviewStandard.TYPE_CAR);

        // 获取当前问卷答案
        List<ReviewDetailsResult> carResults = reviewDetailsResultService.findByTaskId(carReviewTask.getId());

        // 车型审查
        setCarDataStatistics(carStandardTree,carResults,carReviewTask);

        response.setCarReviewTask(carReviewTask);

        Task task = taskMapper.getBySubtaskId(carReviewTask.getId());

        if (task.getModelTestTaskId() != null) {
            response.setTestScenarioList(testScenarioMapper.selectAll());
        }

        return response;
    }

    @Override
    public void doStartReview(CarReviewTask carReviewTask) {
        throw new ServiceException("不能开始一个已确认的任务", HttpStatus.ERROR);
    }

    @Override
    public void doConfirmReview(CarReviewTask carReviewTask) {
        throw new ServiceException("不能确认一个已确认的任务", HttpStatus.ERROR);
    }

    @Override
    public void doSubmitReview(CarReviewTask carReviewTask, List<String> imagesUrl, List<Long> testScenarioList) {

        // 1. 填充review_submit_time
        carReviewTask.setTaskEndTime(new Date());

        // 2. 修改任务状态为 FINISH
        carReviewTask.setTaskStatus(CarReviewTask.TASK_STATUS_FINISH);

        // 3. 填写当前用户id
        // 获取当前登陆人
        SysUser loginUser = SecurityUtils.getLoginUser().getUser();
        carReviewTask.setSubmitId(loginUser.getUserId());
        carReviewTask.setSubmitName(loginUser.getNickName());

        // 4. 保存签名图片地址
        String joinImageUrl = StringUtils.join(imagesUrl,",");
        carReviewTask.setImagesUrl(joinImageUrl);

        // 5.再找带该子任务所属的总任务，将总任务的状态由 RUNNING 改回 RUNNABLE
        Task task = taskMapper.selectById(taskMapper.findByCarReviewTaskId(carReviewTask.getId()));
        taskStateFacade.doSubmit(task);

        carReviewTaskMapper.updateById(carReviewTask);
        taskMapper.updateById(task);

        // 总任务包含车型试验，添加任务-场景关联信息
        if (CollUtil.isNotEmpty(testScenarioList)) {
            for (Long sceneId : testScenarioList) {
                TaskScenarioRelation taskScenario = TaskScenarioRelation.builder()
                        .taskId(task.getId())
                        .testScenarioId(sceneId)
                        .build();
                taskScenarioRelationMapper.insert(taskScenario);
            }
        }

    }

    private void setCarDataStatistics(List<ReviewStandard> standards, List<ReviewDetailsResult> results, CarReviewTask carReviewTask) {
        Map<Long, ReviewDetailsResult> resultMap = results.stream().collect(Collectors.toMap(ReviewDetailsResult::getReviewDetailsId, Function.identity(), (v1, v2) -> v1));

        List<TaskUserRelation> relationList = taskUserRelationMapper.selectQTeamMembers(taskMapper.findByCarReviewTaskId(carReviewTask.getId()));

        BigDecimal standardPass = new BigDecimal((calculateStandardsPass(standards, resultMap) / (double) carReviewTask.getStandards())*100);
        carReviewTask.setStandardsPass(standardPass.setScale(1,BigDecimal.ROUND_UP).doubleValue());
        BigDecimal detailsPass = new BigDecimal((DataStatisticsUtils.calculateDetailsPass(standards, resultMap) / (double) carReviewTask.getDetails())*100);
        carReviewTask.setDetailsPass(detailsPass.setScale(1,BigDecimal.ROUND_UP).doubleValue());
        carReviewTask.setPerson(relationList.size());

    }


    @Override
    public void afterPropertiesSet() throws Exception {
        strategyCarReviewTaskContext.putResource(CarReviewTask.TASK_STATUS_SIGNED, applicationContext.getBean(this.getClass()));
    }
}
