package com.ruoyi.service.impl;

import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.ruoyi.common.constant.HttpStatus;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.domain.ModelTestTask;
import com.ruoyi.domain.Task;
import com.ruoyi.service.*;
import com.ruoyi.web.request.TaskStartRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.Objects;

@Component
public abstract class TaskStateAdapter implements TaskState {

    @Autowired
    private SystemReviewTaskService systemReviewTaskService;

    @Autowired
    private CarReviewTaskService carReviewTaskService;

    @Autowired
    private TaskService taskService;

    @Autowired
    private ModelTestTaskService modelTestTaskService;

    @Override
    public void doStart(Task task, TaskStartRequest request) {

        task.setTaskStatus(Task.TASK_STATUS_PENDING);
        task.setRunStatus(Task.RUN_STATUS_STATUS_RUNNING);

        if (Objects.equals(task.getNextNode(), Task.SUB_TASK_SRAIF)) {
            systemReviewTaskService.start(task.getSystemReviewTaskId());
            taskService.updateById(task);
        } else if (Objects.equals(task.getNextNode(), Task.SUB_TASK_GRFIS)) {
            carReviewTaskService.startReviewTask(task.getCarReviewTaskId());
            taskService.updateById(task);
        } else if (Objects.equals(task.getNextNode(), Task.SUB_TASK_TRFIS)) {
            modelTestTaskService.startModelTestTask(task.getModelTestTaskId());
            // 更新车型试验的试验方案ID
            modelTestTaskService.update(new ModelTestTask(),
                    new LambdaUpdateWrapper<ModelTestTask>()
                            .set(ModelTestTask::getTestSchemeId, request.getTestSchemeId())
                            .eq(ModelTestTask::getId, task.getModelTestTaskId()));
            taskService.updateById(task);
        } else {
            throw new ServiceException("已没有可执行的子任务节点", HttpStatus.ERROR);
        }

        if (task.getPointer() + 1 <= task.getMaxPointer() ) {
            task.setPointer(task.getPointer() + 1);
        }

        if (task.getTaskBeginTime() == null) {
            task.setTaskBeginTime(new Date());
        }
        taskService.updateById(task);

    }
}
